"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const system_1 = require("../system");
const request_1 = require("./request");
const response_1 = require("./response");
const errors_1 = require("./errors");
/**
 * The base endpoint class implemented by all endpoints.
 *
 * @export
 * @class Endpoint
 */
class Endpoint {
    constructor() {
        /**
         * An array of middleware objects.
         *
         * @type {Middleware[]}
         * @memberof Endpoint
         */
        this.middleware = [];
        /**
         * An array of references to processRequest handlers.
         *
         * @type {any[]}
         * @memberof Endpoint
         */
        this.before = [];
        /**
         * An array of references to processResponse handlers.
         *
         * @type {any[]}
         * @memberof Endpoint
         */
        this.after = [];
    }
    /**
     * The base handler registered with serverless. Returns an async function
     * that handles standard serverless handler arguments. The returned
     * function mounts all middleware and a verb-based handler function based
     * on the request method. Middleware request processors are called first,
     * then the verb-based handler, then middleware response processors. If an
     * error is thrown, it is converted to a response object and sent.
     *
     * @throws {HTTPError}
     * @returns async (evt: APIGatewayEvent, ctx: Context)
     * @memberof Endpoint
     */
    handler() {
        return (evt, ctx) => __awaiter(this, void 0, void 0, function* () {
            let res = new response_1.Response();
            let req = new request_1.Request(evt);
            let verb = evt.httpMethod.toLowerCase();
            let callstack = [
                ...this.before,
                this[verb],
                ...this.after.reverse()
            ];
            try {
                for (let call of callstack) {
                    yield call(req, res);
                }
            }
            catch (exc) {
                system_1.log.error(exc);
                if (exc instanceof errors_1.HTTPError) {
                    res = exc.toResponse(res);
                }
                else {
                    let err = new errors_1.InternalError(exc.message);
                    res = err.toResponse(res);
                }
            }
            return res.toLambdaResponse();
        });
    }
    /**
     * Adds a middleware object to the stack.
     *
     * @param {Middleware} middleware
     * @memberof Endpoint
     */
    register(middleware) {
        this.middleware.push(middleware);
        this.before.push(middleware.processRequest);
        this.after.push(middleware.processResponse);
    }
    /**
     * Base implementation of GET.
     *
     * @param {Request} req
     * @param {Response} res
     * @throws {MethodNotAllowed}
     * @returns {Promise<any>}
     * @memberof Endpoint
     */
    get(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new errors_1.MethodNotAllowed();
        });
    }
    /**
     * Base implementation of PUT.
     *
     * @param {Request} req
     * @param {Response} res
     * @throws {MethodNotAllowed}
     * @returns {Promise<any>}
     * @memberof Endpoint
     */
    put(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new errors_1.MethodNotAllowed();
        });
    }
    /**
     * Base implementation of POST.
     *
     * @param {Request} req
     * @param {Response} res
     * @throws {MethodNotAllowed}
     * @returns {Promise<any>}
     * @memberof Endpoint
     */
    post(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new errors_1.MethodNotAllowed();
        });
    }
    /**
     * Base implementation of DELETE.
     *
     * @param {Request} req
     * @param {Response} res
     * @throws {MethodNotAllowed}
     * @returns {Promise<any>}
     * @memberof Endpoint
     */
    delete(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new errors_1.MethodNotAllowed();
        });
    }
}
exports.Endpoint = Endpoint;
//# sourceMappingURL=endpoint.js.map