"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const response_1 = require("./response");
const ERRORS = {
    BadRequest: 'The request was malformed.',
    Unauthorized: 'The request failed authorization.',
    NotFound: 'The request resource was not found.',
    MethodNotAllowed: 'The requested method is not allowed.',
    InternalError: 'An internal error occured.',
    Forbidden: 'Insufficient permissions to access the resource.',
};
/**
 * The base error class. Implements a standard Javascript Error object that
 * can be sent as an HTTP response.
 *
 * @export
 * @class HTTPError
 * @extends {Error}
 */
class HTTPError extends Error {
    /**
     * Creates an instance of HTTPError.
     * @param {number} [code]
     * @param {string} [message]
     * @memberof HTTPError
     */
    constructor(code, message) {
        super(message);
        /**
         * The status code of the response.
         *
         * @memberof HTTPError
         */
        this.code = 500;
        /**
         * The message associated with the response code.
         *
         * @memberof HTTPError
         */
        this.message = ERRORS.InternalError;
        // https://bit.ly/2jFnR9i
        Object.setPrototypeOf(this, HTTPError.prototype);
        this.code = code || this.code;
        this.message = message || this.message;
    }
    /**
     * Returns a response object suitable for serverless.
     *
     * @returns {Response}
     * @memberof HTTPError
     */
    toResponse(res) {
        let body = {
            code: this.code,
            message: this.message
        };
        const errRes = new response_1.Response(this.code, body);
        if (res) {
            errRes.headers = res.headers;
        }
        return errRes;
    }
}
exports.HTTPError = HTTPError;
/**
 * An implementation of a Bad Request error.
 *
 * @export
 * @class BadRequest
 * @extends {HTTPError}
 */
class BadRequest extends HTTPError {
    constructor(message = ERRORS.BadRequest) {
        super(400, message);
        Object.setPrototypeOf(this, BadRequest.prototype);
    }
}
exports.BadRequest = BadRequest;
/**
 * An implementation of an Unauthorized error.
 *
 * @export
 * @class Unauthorized
 * @extends {HTTPError}
 */
class Unauthorized extends HTTPError {
    constructor(message = ERRORS.Unauthorized) {
        super(401, message);
        Object.setPrototypeOf(this, Unauthorized.prototype);
    }
}
exports.Unauthorized = Unauthorized;
/**
 * An implementation of a Forbidden error.
 *
 * @export
 * @class Forbidden
 * @extends {HTTPError}
 */
class Forbidden extends HTTPError {
    constructor(message = ERRORS.Forbidden) {
        super(403, message);
        Object.setPrototypeOf(this, Forbidden.prototype);
    }
}
exports.Forbidden = Forbidden;
/**
 * An implementation of a Not Found error.
 *
 * @export
 * @class NotFound
 * @extends {HTTPError}
 */
class NotFound extends HTTPError {
    constructor(message = ERRORS.NotFound) {
        super(404, message);
        Object.setPrototypeOf(this, NotFound.prototype);
    }
}
exports.NotFound = NotFound;
/**
 * An implementation of a Method Not Allowed error.
 *
 * @export
 * @class MethodNotAllowed
 * @extends {HTTPError}
 */
class MethodNotAllowed extends HTTPError {
    constructor(message = ERRORS.MethodNotAllowed) {
        super(405, message);
        Object.setPrototypeOf(this, MethodNotAllowed.prototype);
    }
}
exports.MethodNotAllowed = MethodNotAllowed;
/**
 * An implementation of an Internal Error error.
 *
 * @export
 * @class InternalError
 * @extends {HTTPError}
 */
class InternalError extends HTTPError {
    constructor(message = ERRORS.InternalError) {
        super(500, message);
        Object.setPrototypeOf(this, InternalError.prototype);
    }
}
exports.InternalError = InternalError;
//# sourceMappingURL=errors.js.map