"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = require("jsonwebtoken");
const jwkToPem = require("jwk-to-pem");
const system_1 = require("../system");
const middleware_1 = require("./middleware");
const errors_1 = require("../http/errors");
/**
 * Extracts JWT from an Authorization header.
 *
 * @param {string} [headerValue]
 * @returns {string}
 */
function getBearerToken(headerValue) {
    if (!headerValue || !headerValue.includes('Bearer')) {
        throw new errors_1.Unauthorized('Authorization failed due to missing credentials.');
    }
    return headerValue.split(' ')[1];
}
/**
 * RBAC middleware via user-provided evaluator functions.
 *
 * @export
 * @class Roles
 * @extends {Middleware}
 */
class Roles extends middleware_1.Middleware {
    /**
     * Creates an instance of Roles.
     * @param {Array<(req: Request, res: Response) => Promise<any>>} evaluators
     * @memberof Roles
     */
    constructor(evaluators) {
        super();
        /**
         * Executes each evaluator against the current request and response.
         *
         * @param {Request} req
         * @param {Response} res
         * @throws {Unauthorized}
         * @returns {Promise<any>}
         * @memberof Roles
         */
        this.processRequest = (req, res) => __awaiter(this, void 0, void 0, function* () {
            for (let evaluator of this.evaluators) {
                let allowed = yield evaluator(req, res);
                if (!allowed) {
                    throw new errors_1.Unauthorized('You do not have permissions to access this resource.');
                }
            }
        });
        this.evaluators = evaluators;
    }
}
exports.Roles = Roles;
/**
 * JWT processing middleware.
 *
 * @export
 * @class JWT
 * @extends {Middleware}
 */
class JWT extends middleware_1.Middleware {
    /**
     * Creates an instance of JWT.
     * @param {string} secret
     * @param {string} [algorithm='HS256']
     * @memberof JWT
     */
    constructor(secret, algorithm = 'HS256') {
        super();
        /**
         * Extracts and verifies the JWT from the Authorization header.
         *
         * @param {Request} req
         * @param {Response} res
         * @throws {Unauthorized}
         * @returns {Promise<any>}
         * @memberof JWT
         */
        this.processRequest = (req, res) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                let token = getBearerToken(req.headers.Authorization);
                let options = {
                    algorithms: [this.algorithm]
                };
                try {
                    req.auth = jwt.verify(token, this.secret, options);
                    resolve();
                }
                catch (exc) {
                    system_1.log.error(exc);
                    let err = new errors_1.Unauthorized('Authorization failed: Invalid credentials.');
                    reject(err);
                }
            });
        });
        this.secret = secret;
        this.algorithm = algorithm;
    }
}
exports.JWT = JWT;
/**
 * JWK processing middleware.
 *
 * @export
 * @class JWK
 * @extends {Middleware}
 */
class JWK extends middleware_1.Middleware {
    /**
     * Creates an instance of JWK.
     * @param {object[]} jwks
     * @throws {InternalError}
     * @memberof JWK
     */
    constructor(jwks) {
        super();
        /**
         * Extracts and verifies the JWT from the Authorization header.
         *
         * @param {Request} req
         * @param {Response} res
         * @throws {Unauthorized}
         * @returns {Promise<any>}
         * @memberof JWK
         */
        this.processRequest = (req, res) => __awaiter(this, void 0, void 0, function* () {
            // TODO: implement full validation for Cognito
            // https://amzn.to/2fo77UI
            let token = getBearerToken(req.headers.Authorization);
            for (let key of this.keys) {
                let pem = jwkToPem(key);
                try {
                    req.auth = jwt.verify(token, pem);
                    return;
                }
                catch (exc) {
                    // keep trying;
                }
            }
            system_1.log.error('No keys were able to validate the token.');
            throw new errors_1.Unauthorized('Authorization failed due to invalid credentials.');
        });
        this.keys = jwks;
        if (this.keys.length < 1) {
            system_1.log.debug('Empty key array provided.');
            throw new errors_1.InternalError('Unable to read JWKS.');
        }
    }
}
exports.JWK = JWK;
//# sourceMappingURL=authorization.js.map